﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.ComponentModel;
using System.Web.UI.WebControls;
using System.Collections.Specialized;


namespace MyControls {

    public enum InactiveLinkMode { Ignore, Hide, Disable }
    public enum PageIndexSource { RouteParameter, QueryString }

    public class DataPagerExtender : Control {
        private const int DEFAULT_PAGE_SIZE = 10;
        private const string DEFAULT_COLLECTION_KEY = "PageIndex";
        private const string DESIGN_TIME_HTML = "<table cellpadding=4 cellspacing=0 style=\"font:messagebox;color:buttontext;background-color:buttonface;border: solid 1px;border-top-color:buttonhighlight;border-left-color:buttonhighlight;border-bottom-color:buttonshadow;border-right-color:buttonshadow\"><tr><td nowrap><span style=\"font-weight:bold\">{0}</span> - {1}</td></tr><tr><td>{2}</td></tr></table>";

        // Constructor

        public DataPagerExtender() {
            this.PageSize = DEFAULT_PAGE_SIZE;
            this.CollectionKey = DEFAULT_COLLECTION_KEY;
            this.InactiveLinkMode = InactiveLinkMode.Ignore;
            this.PageIndexSource = PageIndexSource.RouteParameter;
        }

        // Event handler overrides

        protected override void OnLoad(EventArgs e) {
            base.OnLoad(e);

            // Find target control
            this.TargetControl = this.NamingContainer.FindControl(this.TargetControlID) as IPageableItemContainer;
            if (this.TargetControl == null) throw new ArgumentException("Target control does not exist or does not implement IPageableItemContainer.");

            // Setup paging
            this.TargetControl.SetPageProperties(this.StartRowIndex, this.PageSize, true);

            // Get total rows when available
            this.TargetControl.TotalRowCountAvailable += new EventHandler<PageEventArgs>(TargetControl_TotalRowCountAvailable);
        }

        protected override void Render(HtmlTextWriter writer) {
            if (!this.DesignMode) return;
            writer.Write(DESIGN_TIME_HTML, "DataPagerExtender", this.ID, "");
        }

        // Helper methods

        protected void SetupPagingLinks() {
            if (this.PageIndex == 1) {
                this.SetupPagingLinkControl(this.FirstLinkID, 0);
                this.SetupPagingLinkControl(this.PreviousLinkID, 0);
            }
            else {
                this.SetupPagingLinkControl(this.FirstLinkID, 1);
                this.SetupPagingLinkControl(this.PreviousLinkID, this.PageIndex - 1);
            }
            if (this.PageIndex == this.TotalPages) {
                this.SetupPagingLinkControl(this.NextLinkID, 0);
                this.SetupPagingLinkControl(this.LastLinkID, 0);
            }
            else {
                this.SetupPagingLinkControl(this.NextLinkID, this.PageIndex + 1);
                this.SetupPagingLinkControl(this.LastLinkID, this.TotalPages);
            }
        }

        protected void SetupPagingLinkControl(string controlId, int pageIndex) {
            // Do nothing when link is to be ignored and is inactive
            if (pageIndex == 0 && this.InactiveLinkMode == InactiveLinkMode.Ignore) return;

            // Find link control
            var link = FindPagingLink(controlId);
            if (link == null) return;

            if (pageIndex == 0) {
                // Link is inactive
                if (this.InactiveLinkMode == InactiveLinkMode.Hide) link.Visible = false;
                else { link.Enabled = false; }
                return;
            }

            // Link is active - get page URL
            link.NavigateUrl = this.GetUrlForPage(pageIndex);
        }

        protected string GetUrlForPage(int pageIndex) {
            if (this.PageIndexSource == PageIndexSource.RouteParameter) {
                // Get URL via routing engine
                var dict = new System.Web.Routing.RouteValueDictionary(this.Page.RouteData.Values);
                dict[this.CollectionKey] = pageIndex;
                return this.Page.RouteData.Route.GetVirtualPath(this.Page.Request.RequestContext, dict).VirtualPath;
            }
            else {
                // Setup new query string values
                var qs = new NameValueCollection(this.Page.Request.QueryString);
                if (pageIndex == 1) qs.Remove(this.CollectionKey);
                else qs.Set(this.CollectionKey, pageIndex.ToString());

                // Construct new URL using modified query string values
                var qsb = new System.Text.StringBuilder(this.Page.Request.AppRelativeCurrentExecutionFilePath);
                qsb.Append("?");
                foreach (var key in qs.AllKeys) {
                    qsb.AppendFormat("{0}={1}&", key, HttpUtility.UrlEncode(qs[key]));
                }
                var url = qsb.ToString();
                url = url.TrimEnd('?', '&');
                return url;
            }
        }

        protected HyperLink FindPagingLink(string controlId) {
            if (string.IsNullOrWhiteSpace(controlId)) return null;
            var link = this.NamingContainer.FindControl(controlId) as HyperLink;
            if (link == null) link = (this.TargetControl as Control).FindControl(controlId) as HyperLink;
            return link;
        }

        void TargetControl_TotalRowCountAvailable(object sender, PageEventArgs e) {
            // Get total row count
            this.TotalRowCount = e.TotalRowCount;

            // Setup paging links
            this.SetupPagingLinks();
        }

        // Properties

        [Category("Behavior"), Description("ID of IPageableItemContainer control to be paged.")]
        [IDReferenceProperty(typeof(IPageableItemContainer))]
        public string TargetControlID {
            get { return (string)this.ViewState["TargetControlID"]; }
            set { this.ViewState["TargetControlID"] = value; }
        }

        protected IPageableItemContainer TargetControl { get; set; }

        #region Configuration properties

        [Category("Behavior"), Description("Maximum number of items on page.")]
        [DefaultValue(DEFAULT_PAGE_SIZE)]
        public int PageSize {
            get { return (int)this.ViewState["PageSize"]; }
            set { this.ViewState["PageSize"] = value; }
        }

        [Category("Behavior"), Description("Name of key in collection to read current page index from. Collection is specified by PageIndexSource property.")]
        [DefaultValue(DEFAULT_COLLECTION_KEY)]
        public string CollectionKey {
            get { return (string)this.ViewState["CollectionKey"]; }
            set { this.ViewState["CollectionKey"] = value; }
        }

        [Category("Behavior"), Description("Defines what should happen with links being inactive in given context.")]
        [DefaultValue(InactiveLinkMode.Ignore)]
        public InactiveLinkMode InactiveLinkMode {
            get { return (InactiveLinkMode)this.ViewState["InactiveLinkMode"]; }
            set { this.ViewState["InactiveLinkMode"] = value; }
        }

        [Category("Behavior"), Description("Defines where the current page index is to be read from.")]
        [DefaultValue(PageIndexSource.RouteParameter)]
        public PageIndexSource PageIndexSource {
            get { return (PageIndexSource)this.ViewState["PageIndexSource"]; }
            set { this.ViewState["PageIndexSource"] = value; }
        }

        [Category("Paging Controls"), Description("ID of HyperLink control pointing to first page.")]
        [IDReferenceProperty(typeof(HyperLink))]
        public string FirstLinkID {
            get { return (string)this.ViewState["FirstLinkID"]; }
            set { this.ViewState["FirstLinkID"] = value; }
        }

        [Category("Paging Controls"), Description("ID of HyperLink control pointing to previous page.")]
        [IDReferenceProperty(typeof(HyperLink))]
        public string PreviousLinkID {
            get { return (string)this.ViewState["PreviousLinkID"]; }
            set { this.ViewState["PreviousLinkID"] = value; }
        }

        [Category("Paging Controls"), Description("ID of HyperLink control pointing to next page.")]
        [IDReferenceProperty(typeof(HyperLink))]
        public string NextLinkID {
            get { return (string)this.ViewState["NextLinkID"]; }
            set { this.ViewState["NextLinkID"] = value; }
        }

        [Category("Paging Controls"), Description("ID of HyperLink control pointing to last page.")]
        [IDReferenceProperty(typeof(HyperLink))]
        public string LastLinkID {
            get { return (string)this.ViewState["LastLinkID"]; }
            set { this.ViewState["LastLinkID"] = value; }
        }

        #endregion

        #region Paging-related properties

        [Browsable(false)]
        public int PageIndex {
            get {
                string pageIndexString;
                if (this.PageIndexSource == PageIndexSource.QueryString) {
                    pageIndexString = this.Page.Request.QueryString[this.CollectionKey];
                }
                else {
                    pageIndexString = this.Page.RouteData.Values[this.CollectionKey] as string;
                }
                if (string.IsNullOrWhiteSpace(pageIndexString)) pageIndexString = "1";
                return int.Parse(pageIndexString);
            }
        }

        [Browsable(false)]
        public int StartRowIndex {
            get {
                return (this.PageIndex - 1) * this.PageSize;
            }
        }

        [Browsable(false)]
        public int TotalRowCount { get; private set; }

        [Browsable(false)]
        public int TotalPages {
            get {
                return (int)Math.Ceiling(this.TotalRowCount / (decimal)this.PageSize);
            }
        }

        #endregion

    }

}
